import java.util.List;
import java.util.ArrayList;


public class HanoiSolver
	 extends Hanoi
{

	 // initialize board with `N' sorted disks on left peg
	 HanoiSolver(int N) {
		  super(N);
	 }
	 
	 // compute sequence of moves to solution
	 public List<String> solve(boolean showBoard) {
		  return move(N, PegPosition.LEFT, PegPosition.MIDDLE, PegPosition.RIGHT,
						  showBoard);
	 }

	 // display sequence of moves to solution
	 public void solve() {
		  move(N, PegPosition.LEFT, PegPosition.MIDDLE, PegPosition.RIGHT);
	 }
	 
	 // move `n' top disks from `source' peg to `destination' peg via `spare' peg
	 public List<String> move(int n,
									  PegPosition source,
									  PegPosition destination,
									  PegPosition spare,
									  boolean showBoard) {
		  List<String> moves = new ArrayList<>();
		  if (n == 1) {
				// base case
				moves.add(move(source, destination, showBoard));
		  } else {
				// recursively move n - 1 to spare
				moves.addAll(move(n - 1, source, spare, destination, showBoard));
				// move largest disk
				moves.addAll(move(1, source, destination, spare, showBoard));
				// recursively move n - 1 to destination
				moves.addAll(move(n - 1, spare, destination, source, showBoard));
		  }
		  return moves;
	 }

	 // move `n' top disks from `source' peg to `destination' peg via `spare' peg
	 // and print board after each move
	 public void move(int n,
							PegPosition source,
							PegPosition destination,
							PegPosition spare) {
		  if (n == 1) {
				// base case
				move(source, destination);
				System.out.println(this);
		  } else {
				// recursively move n - 1 to spare
				move(n - 1, source, spare, destination);
				// move largest disk
				move(1, source, destination, spare);
				// recursively move n - 1 to destination
				move(n - 1, spare, destination, source);
		  }
	 }
	 
	 public static void main(String[] args) {
		  int nDisks = 3;
		  // if the first command-line argument is an integer
		  // use it as the number of disks
		  if (args.length > 0) {
				try {
					 nDisks = Integer.parseInt(args[0]);
				} catch (NumberFormatException e) {
					 // Just ignore argument
				}
		  }
		  HanoiSolver h = new HanoiSolver(nDisks);
		  System.out.println(h); // print initial state
		  h.solve(); // solve
	 }
}
